Onchain (onchain v0.5.0)

Copy Markdown View Source

Shared Ethereum/blockchain library providing read and write capabilities.

Uses signet as the sole Ethereum dependency for RPC calls, ABI encoding, transaction signing, and cryptographic operations.

Discovery

Use Onchain.describe/0 for a module overview, Onchain.describe/1 for function listings, and Onchain.describe/2 for full function details.

Summary

Functions

Return the list of modules registered with this library.

Return a Level 1 overview of all modules in this library.

Return Level 2 function list for a module (by full atom or short name).

Return Level 3 function detail (or nil if not found).

Functions

__descripex_modules__()

@spec __descripex_modules__() :: [module()]

Return the list of modules registered with this library.

describe()

@spec describe() :: [map()]

Return a Level 1 overview of all modules in this library.

describe(mod_or_short)

@spec describe(module() | atom()) :: [map()]

Return Level 2 function list for a module (by full atom or short name).

describe(mod_or_short, func_name)

@spec describe(module() | atom(), atom()) :: map() | nil

Return Level 3 function detail (or nil if not found).